import numpy as np

def diff(x, psi_g):
    if psi_g.dtype == complex:
        return diff(x, psi_g.real) + 1j * diff(x, psi_g.imag)
    dx = x[1] - x[0]
    diffvalues = (psi_g[1:] - psi_g[:-1]) / dx
    dpsidx_g = np.interp(x, x[:-1] + dx / 2.0, diffvalues)
    return dpsidx_g

def _stitch(xvalues, func):
    yvalues = [func(xvalues[0], 0)]
    branch = 0
    for x in xvalues[1:]:
        ycandidates = np.array([func(x, branch + j) for j in [-1, 0, 1]])
        yarg = np.argmin(np.abs(yvalues[-1] - ycandidates))
        branch += -1 + yarg
        yvalues.append(ycandidates[yarg])
    return yvalues

def stitch(x, func):
    i0 = len(x) // 2
    y1 = _stitch(x[i0 - 1::-1], func)[::-1]
    y2 = _stitch(x[i0:], func)
    y = np.array(y1 + y2)
    assert len(y) == len(x), (len(y), len(x))
    return y

def rootbranch(rho, i, power):
    return rho**(1.0 / power) * np.exp(i * 2j * np.pi / power)
def cuberoot(rho, i):
    return rootbranch(rho, i, 3)
def sixthroot(rho, i):
    return rootbranch(rho, i, 6)
def logarithm(rho, i):
    return np.log(rho) + i * 2j * np.pi

def epsx_complex(rho, theta):
    #def cuberoot(rho, branch):
    #    return rho**(1.0 / 3.0) * np.exp(branch * 2j * np.pi / 3.0)
    vx = -0.98474502184269641 * stitch(rho * np.exp(-1j * theta), cuberoot)
    epsx = vx * 3.0 / 4.0
    return epsx, vx

def epsx(n):
    eps = -3.0 / 4.0 * (3.0 / np.pi)**(1.0 / 3.0) * n**(1.0 / 3.0)
    dedn = -1.0 / 4.0 * (3.0 / np.pi)**(1.0 / 3.0) * n**(-2.0 / 3.0)
    v = eps + dedn * n
    return eps, v

C0I = 0.238732414637843
C1 = -0.45816529328314287
CC1 = 1.9236610509315362
CC2 = 2.5648814012420482
IF2 = 0.58482236226346462

gamma = 0.031091
alpha1 = 0.21370
beta1 = 7.5957
beta2 = 3.5876
beta3 = 1.6382
beta4 = 0.49294

def epsc_complex(n, theta):
    invn = 3.0 / (1e-20 + 4.0 * np.pi * n * np.exp(-1j * theta))
    rtrs = stitch(invn, sixthroot)
    Q0 = -2.0 * gamma * (1.0 + alpha1 * rtrs**2)
    Q1 = 2.0 * gamma * rtrs * (beta1 +
                               rtrs * (beta2 +
                                       rtrs * (beta3 +
                                               rtrs * beta4)))
    eps_c = Q0 * stitch(1.0 + 1.0 / Q1, logarithm)

    dQ1drs = gamma * (beta1 / rtrs + 2.0 * beta2 +
                      rtrs * (3.0 * beta3 + 4.0 * beta4 * rtrs))
    dedrs = -2.0 * gamma * alpha1 * eps_c / Q0 - Q0 * dQ1drs / (Q1 * (Q1 
                                                                      + 1.0))

    v = eps_c - rtrs**2 * dedrs / 3.0
    return eps_c, v
    

def epsc(n):
    rs = (C0I / n)**(1.0 / 3.0)
    rtrs = np.sqrt(rs)
    Q0 = -2.0 * gamma * (1.0 + alpha1 * rtrs**2)
    Q1 = 2.0 * gamma * rtrs * (beta1 +
                               rtrs * (beta2 +
                                       rtrs * (beta3 +
                                               rtrs * beta4)))
    eps_c = Q0 * np.log(1.0 + 1.0 / Q1)

    dQ1drs = gamma * (beta1 / rtrs + 2.0 * beta2 +
                      rtrs * (3.0 * beta3 + 4.0 * beta4 * rtrs))
    dedrs = -2.0 * gamma * alpha1 * eps_c / Q0 - Q0 * dQ1drs / (Q1 * (Q1 + 1.0))

    v = eps_c - rs * dedrs / 3.0
    return eps_c, v

def xc_lda(n):
    epsx, vx = eps_x(n)
    epsc, vc = eps_c(n)
    return epsx + epsc, vx + vc

def xc_lda_complex(n, theta):
    epsx, vx = epsx_complex(n, theta)
    epsc, vc = epsc_complex(n, theta)
    return epsx + epsc, vx + vx

def expi(x):
    return np.exp(1j * x)

def get_kinetic2(x, theta):
    N = len(x)
    a = -x[0]
    dx = x[1] - x[0]
    T = np.zeros((N, N), dtype=complex)
    n = (N - 1) // 2
    
    Q = np.zeros((N, N))
    Q += np.arange(len(x))[None, :]
    Q -= np.arange(len(x))[:, None]
    Q *= 2 * np.pi / N
    
    lval = np.arange(1, n + 1, dtype=float)
    l2val = lval**2

    
    for q in range(N):
       for j in range(N):
           T[q, j] = (np.cos(lval * Q[q, j]) * l2val).sum()
    T *= 4 * np.exp(-2j * theta) / N * (np.pi / N / dx)**2
    #print 'var', np.var(np.diag(T))
    #print 'var', np.var(np.diag(T, 1))
    return T
    
stencils = [[0.],
            [-2., 1.],
            [-5./2., 4./3., -1./12.],
            [-49./18., 3./2., -3./20., 1./90.],
            [-205./72., 8./5., -1./5., 8./315., -1./560.],
            [-5269./1800., 5./3., -5./21., 5./126., -5./1008., 1./3150.],
            [-5369./1800., 12./7., -15./56., 10./189., -1./112., 2./1925.,
             -1./16632.]]


def get_kinetic(x, order=1, periodic=False):
    N = len(x)
    dx = x[1] - x[0]
    T = np.zeros((N, N), dtype=complex)
    Tflat = T.ravel()

    stencil = stencils[order]

    for i, val in enumerate(stencil):
        Tflat[i::N + 1] = val
        Tflat[i * N::N + 1] = val
    if periodic:
        if order > 1:
            raise NotImplementedError
        T[-1, 0] = 1.0
        T[0, -1] = 1.0
    else:
        for tmp in [T[:, -1], T[:, 0], T[0, :], T[-1, :]]:
            tmp[:] = 0.0
    T *= -0.5 / dx**2
    return T

def oldget_kinetic(x, periodic=False):
    N = len(x)
    dx = x[1] - x[0]
    T = np.zeros((N, N), dtype=complex)
    Tflat = T.ravel()
    Tflat[0::N+1] = -2.0
    Tflat[1::N+1] = 1.0
    Tflat[N::N+1] = 1.0
    if periodic:
        T[-1, 0] = 1.0
        T[0, -1] = 1.0
    else:
        for tmp in [T[:, -1], T[:, 0], T[0, :], T[-1, :]]:
            tmp[:] = 0.0
    T *= -0.5 / dx**2
    return T



def sinsquared(x, theta):
    x1 = np.linspace(-np.pi / 2.0, np.pi / 2.0, len(x))
    return np.sin(x1 * np.exp(1j * theta))**2
    
def harmonic(x, n=2):
    alpha = 2.0
    v = alpha * x**n
    return v

def stark(x, E=0.1):
    return E * x

def gausswell(x, depth=4.0, barrier=4.0, iwidth=2.0, dist=2.0):
    t0 = depth
    a0 = barrier
    c0 = iwidth
    d0 = dist
    
    def gauss(x, alpha):
        return np.exp(-alpha * x**2)
    
    g1 = gauss(x - d0, c0) * a0
    g2 = gauss(x + d0, c0) * a0
    g3 = -t0 * gauss(x, c0)
    v = g1 + g2 + g3
    return v

def barriers(x, theta=None):
    a0 = 4.0
    b0 = 0.5
    c0 = 2.0
    d0 = 4.0
    if theta is None:
        dtype = float
        eith = 1.0
    else:
        eith = np.exp(1j*theta)
        dtype = complex
    v = a0 * (-1.0/(1.0 + np.exp(-2.0*c0*(x*eith-d0))) + \
                   1.0/(1.0 + np.exp(-2.0*c0*(x*eith+d0))) - \
                   np.exp(-(x*eith)**2/b0))
    return v



class CLDA:
    def __init__(self, theta):
        self.theta = theta

    def xc(self, n):
        epsx, vx = epsx_complex(n, self.theta)
        epsc, vc = epsc_complex(n, self.theta)
        return epsx + epsc, vx + vc
