import numpy as np
from gpaw.xc.gga import gga_constants

from pydft.operator import get_derivative

C1 = -0.45816529328314287
CC1 = 1.9236610509315362
CC2 = 2.5648814012420482
IF2 = 0.58482236226346462


def G(rtrs, gamma, alpha1, beta1, beta2, beta3, beta4):
    Q0 = -2.0 * gamma * (1.0 + alpha1 * rtrs**2)
    Q1 = 2.0 * gamma * rtrs * (beta1 +
                           rtrs * (beta2 +
                                   rtrs * (beta3 +
                                           rtrs * beta4)))
    G1 = Q0 * np.log(1.0 + 1.0 / Q1)
    dQ1drs = gamma * (beta1 / rtrs + 2.0 * beta2 +
                  rtrs * (3.0 * beta3 + 4.0 * beta4 * rtrs))
    dGdrs = -2.0 * gamma * alpha1 * G1 / Q0 - Q0 * dQ1drs / (Q1 * (Q1 + 1.0))
    return G1, dGdrs


def get_rs(n):
    return (0.238732414637843 / n)**(1.0 / 3.0)

class NoXC:
    def xc(self, n):
        return np.zeros_like(n), np.zeros_like(n)

class LDA:
    def x(self, n):
        v = -0.98474502184269641 * n**(1.0 / 3.0)
        return 3.0 / 4.0 * v, v

    def c(self, n):
        g = 0.031091
        a1 = 0.21370
        b1 = 7.5957
        b2 = 3.5876
        b3 = 1.6382
        b4 = 0.49294

        rs = get_rs(n)
        rtrs = np.sqrt(rs)
        eps, dedrs = G(rtrs, g, a1, b1, b2, b3, b4)
        v = eps - rs * dedrs / 3.0
        return eps, v

    def xc(self, n):
        ex, vx = self.x(n)
        ec, vc = self.c(n)
        return ex + ec, vx + vc

class PBE:
    def __init__(self, grid):
        self.D = get_derivative(grid.x, order=1)

    def x(self, n, gradn):
        kappa = 0.804
        mu = 0.2195149727645171
        #beta = 0.06672455060314922


        C0I, C1, C2, C3, CC1, CC2, IF2, GAMMA = gga_constants()

        #if n.dtype == complex:
        #    n = n * np.exp(-1j * theta)
        #    rs = stitch(C0I / (n + 1e-40), cuberoot)
        #    gradn = gradn / phase
        #else:
        #rs = (C0I / n)**(1 / 3.)
        rs = get_rs(n)
        
        # lda part
        ex = C1 / rs
        dexdrs = -ex / rs
        a2 = gradn**2

        # gga part
        s2 = a2 * (C2 * rs / n)**2

        x = 1.0 + mu * s2 / kappa
        Fx = 1.0 + kappa - kappa / x
        ex *= Fx
        vx = ex - rs * dexdrs / 3.
        return ex, vx

    def c(self, n, gradn):
        BETA = 0.06672455060314922
        C0I, C1, C2, C3, CC1, CC2, IF2, GAMMA = gga_constants()

        #if n.dtype == complex:
        #    n = n * np.exp(-1j * theta)
        #    rs = stitch(C0I / (n + 1e-40), cuberoot)
        #    gradn = gradn / phase
        #else:
        rs = get_rs(n)

        # lda part
        ec, decdrs_0 = G(rs**0.5, GAMMA, 0.21370, 7.5957,
                         3.5876, 1.6382, 0.49294)

        decdrs = decdrs_0

        # gga part
        n2 = n**2
        a2 = gradn**2
        t2 = C3 * a2 * rs / n2
        y = -ec / GAMMA

        x = np.exp(y)

        A = np.zeros_like(x)
        import pylab as pl

        zeromask = (y == 0.0)
        indices = np.nonzero(y)
        A[indices] = BETA / (GAMMA * (x[indices] - 1.0))

        At2 = A * t2
        nom = 1.0 + At2
        denom = nom + At2 * At2

        H = GAMMA * np.log(1.0 + BETA * t2 * nom / (denom * GAMMA))
        H[zeromask] = 0.0
        ec += H

        vc = ec - rs * decdrs / 3.
        return ec, vc

    def xc(self, n):
        gradn = np.dot(self.D, n)
        ex, vx = self.x(n, gradn)
        ec, vc = self.c(n, gradn)
        return ex + ec, vx + vc
