import numpy as np

from pydft.operator import get_kinetic
from pydft.xc import LDA

class Grid:
    def __init__(self, x):
        self.x = x
        self.dx = x[1] - x[0]
        self.n = len(x)

    def zeros(self, shape=(), dtype=float):
        return np.zeros((self.n,) + shape, dtype=dtype)

    def int(self, a_x, axis=0):
        return a_x.sum(axis=axis) * self.dx

    def norm(self, psi_x, axis=0):
        return self.int(psi_x**2, axis=axis)

class SoftCoulomb:
    def __init__(self, grid):
        self.grid = grid

    def solve(self, n_g):
        grid = self.grid
        v_g = grid.zeros()
        x = grid.x
        for i, xp in enumerate(x):
            v_g[i] = (n_g / np.sqrt(abs(x - xp)**2 + 1.0)).sum()
        v_g *= grid.dx
        return v_g

g = Grid(np.linspace(-6, 6, 101))
p = SoftCoulomb(g)

T_gg = get_kinetic(g.x, order=6)
vext_g = 2.0 * g.x**2

vH_g = g.zeros()
n_g = g.zeros()
vxc_g = g.zeros()

nstates = 6
ne = 1

f_n = np.zeros(nstates * 2)
f_n[:ne] = 1.0
f_n = f_n.reshape(-1, 2).sum(axis=-1)

mix = 1.0

xc = LDA()

def dot(*args):
    return reduce(np.dot, args)

for i in range(30):
    v_g = vH_g + vxc_g + vext_g
    H_gg = T_gg.copy() + np.diag(v_g)
    eps_n, psi_gn = np.linalg.eigh(H_gg)
    psi_gn = psi_gn[:, :nstates].copy()
    psi_gn /= np.sqrt((psi_gn**2).sum(axis=0) * g.dx)[None, :]

    n_prev = n_g
    n_g = (psi_gn[:, :]**2 * f_n[None, :]).sum(axis=1)
    logerr = np.log10(abs(n_prev - n_g).sum() * (g.dx / ne))
    print 'iter %4d :: logerr %f' % (i, logerr)
    if logerr < -8:
        break
    
    n_g = n_g * mix + n_prev * (1.0 - mix)
    vH_g = p.solve(n_g) # XXX
    exc, vxc_g = xc.xc(n_g)
    

print 'eps', eps_n[:nstates]

Ekin = (eps_n[:nstates] * f_n).sum() - g.int(v_g * n_g)

Eext = g.int(n_g * vext_g)
Ehartree = 0.5 * g.int(n_g * vH_g)
Exc = g.int(exc * n_g)

Etot = Ekin + Ehartree + Exc + Eext

print 'Ekin %8f' % Ekin
print 'EHa  %8f' % Ehartree
print 'Exc  %8f' % Exc
print 'Eext %8f' % Eext
print 'Etot %8f' % Etot


#Epot = (n_g * (vext_g + vH_g)).sum() * g.dx
#Emagic = (eps_n[:nstates] * f_n).sum() - Epot


#print 'energies', Ekin, Eext, Ehartree
#print 'maybe', Emagic

#E = (eps_n[:nstates] * f_n).sum() - (n_g * (vext_g + vH_g)).sum() * g.dx
#print 'energy', E


if 0:
    import pylab as pl
    fig = pl.figure()
    ax1 = fig.add_subplot(221)
    ax2 = fig.add_subplot(222)
    ax3 = fig.add_subplot(223)

    ax1.plot(g.x, n_g)
    for n in range(nstates):
        ax2.plot(g.x, psi_gn[:, n])

    ax3.plot(g.x, vH_g)


    pl.show()
